package net.porkbuns.my_story;

import net.porkbuns.my_story.api.User;
import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.Spinner;

public class EditAvatar extends Activity implements
		AdapterView.OnItemSelectedListener, Button.OnClickListener {
	public final static String AVATAR_FILE_NAME = "avatarData.txt";

	// UI Elements
	private Spinner mHeadSpinner;
	private Spinner mShirtSpinner;
	private Spinner mPantsSpinner;
	private Button mSaveButton;
	private AvatarLayout mAvatarLayout;
	private ProgressDialog pd;
	private boolean dontSave;
	private NoConnectionDialog noConnectionDialog;
	private Handler mHandler = new Handler();

	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.editavatar);

		Bundle extras = getIntent().getExtras();
		if (extras != null) {
			int editOrNot = extras.getInt(
					EditRoomAvatarMain.EDIT_AVATAR_PASS_INFO, 0);
			if (editOrNot != 1) {
				setTitle("My Story > You > Edit Avatar");
			}
		} else {
			setTitle("My Story > Create Avatar");
		}

		// Populate items
		mHeadSpinner = (Spinner) this.findViewById(R.id.head);
		mShirtSpinner = (Spinner) this.findViewById(R.id.shirt);
		mPantsSpinner = (Spinner) this.findViewById(R.id.pants);
		// RelativeLayout rel = (RelativeLayout) findViewById(R.id.main_layout);
		mSaveButton = (Button) this.findViewById(R.id.save_button);
		mSaveButton.setOnClickListener(this);
		// mAvatarLayout = new AvatarLayout(this);
		// rel.addView(mAvatarLayout, new RelativeLayout.LayoutParams(120,
		// 170));
		mAvatarLayout = (AvatarLayout) this.findViewById(R.id.avatar_layout);
		mAvatarLayout.setUser(User.getCurrentUser(this));
		ArrayAdapter<String> headAdapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, mAvatarLayout
						.getHeadItemTitles());
		ArrayAdapter<String> shirtAdapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, mAvatarLayout
						.getShirtItemTitles());
		ArrayAdapter<String> pantsAdapter = new ArrayAdapter<String>(this,
				android.R.layout.simple_spinner_item, mAvatarLayout
						.getPantsItemTitles());

		headAdapter
				.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		shirtAdapter
				.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		pantsAdapter
				.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);

		mHeadSpinner.setAdapter(headAdapter);
		mShirtSpinner.setAdapter(shirtAdapter);
		mPantsSpinner.setAdapter(pantsAdapter);

		User curr = User.getCurrentUser(this);
		mHeadSpinner.setSelection(curr.getHeadPos());
		mShirtSpinner.setSelection(curr.getShirtPos());
		mPantsSpinner.setSelection(curr.getPantsPos());

		mHeadSpinner.setOnItemSelectedListener(this);
		mShirtSpinner.setOnItemSelectedListener(this);
		mPantsSpinner.setOnItemSelectedListener(this);

		mHeadSpinner.requestFocus();
		mAvatarLayout.redraw();
		dontSave = false;
	}

	// handles key presses
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		super.onKeyDown(keyCode, event);
		// NEED TO MAKE THIS DIFFERENT FOR COMING FROM HOME SCREEN OR FROM
		// EDITROOMAVATARMAIN
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK: // go back to EditRoomAvatarMain
			dontSave = true;
			setResult(RESULT_CANCELED);
			finish();
			return true;
		}
		return false;
	}

	@SuppressWarnings("unchecked")
	public void onItemSelected(AdapterView parent, View v, int position, long id) {
		User currentUser = User.getCurrentUser(this);
		int headId = mHeadSpinner.getSelectedItemPosition();
		int shirtId = mShirtSpinner.getSelectedItemPosition();
		int pantsId = mPantsSpinner.getSelectedItemPosition();
		currentUser.setHead(headId);
		currentUser.setShirt(shirtId);
		currentUser.setPants(pantsId);
		mAvatarLayout.redraw();
	}

	@SuppressWarnings("unchecked")
	public void onNothingSelected(AdapterView parent) {

	}

	public void onClick(View v) {
		pd = ProgressDialog.show(this, "Contacting server", "Saving avatar", true, false);
		new SaveAvatarThread().start();
		
	}
	
	protected class SaveAvatarThread extends Thread
	{
		public void run() {
			String response = User.getCurrentUser(EditAvatar.this).save(EditAvatar.this);
			pd.dismiss();
			if (response != null) {
				setResult(RESULT_OK);
				finish();
			}
			else {
				mHandler.post(new ShowDialogThread());
			}
		}
	}
	
	protected class ShowDialogThread extends Thread
	{
	   	public void run() {
	   		noConnectionDialog = new NoConnectionDialog(EditAvatar.this);
			noConnectionDialog.show();
			noConnectionDialog.setTryAgainListener(new TryAgainListener());
			noConnectionDialog.setCancelListener(new CancelListener());
	   	}
	}
	   
	   
	protected class TryAgainListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
			pd = ProgressDialog.show(EditAvatar.this, "Contacting server", "Saving avatar", true, false);
			new SaveAvatarThread().start();
		}
	}
	
	protected class CancelListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
		}
	}

	protected void onPause() {
		super.onPause();
		if (!dontSave) {
			User.getCurrentUser(this).writeAvatarData(this);
		}
	}
}
