package net.porkbuns.my_story;

import net.porkbuns.my_story.api.User;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Gallery.LayoutParams;

// images stored as #'s...referred to as #'s...should be like this in the db.  
// this way, returning the result to replace is easy

public class EditRoom extends Activity {
	// used mImages and gallery from EditRoomAvatarMain
	// getMImages(), setMImages(int, int), getGallery(), setGallery(int, Item)
	private int shelfPosBeingReplaced;
	private final int SELECT_TROPHY_ACTIVITY = 3;
	
	// used as keys for the Bundle so we can pass data back and grab it
	// location in the array of all trophies
	public static final String IMAGE_KEY = "image";
	// the position on the "shelf" that we're going to replace
	public static final String REPLACE_POS_KEY = "replace_bmp_pos";
	
	// result corresponding to the above keys that means no changes made
	public static final String NO_CHANGES = "no_changes";
	
	// tells SelectTrophy.java what position in the thumbID array (mImages) 
	//  the 'select box' is on
	public static final String BITMAP_POS = "bmp_pos";
	
	private ProgressDialog pd;
	private NoConnectionDialog noConnectionDialog;
	private Handler mHandler = new Handler();
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.edit_room);
        
        setTitle("My Story > You > Edit Room");
        
        GridView g = (GridView) findViewById(R.id.myGrid);
		g.setAdapter(new ImageAdapterRoomView(this, User.getCurrentUser(this)));
        
		// set "clicks" on images to take you to the poster/trophy selection grid
		g.setOnItemClickListener(new OnItemClickListener() {
			@SuppressWarnings("unchecked")
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				Intent i = new Intent(EditRoom.this, SelectTrophy.class);
				shelfPosBeingReplaced = position;
				i.putExtra(EditRoom.BITMAP_POS, position);
				startSubActivity(i, SELECT_TROPHY_ACTIVITY);
			}
		});
    }
    
    /**
	 * onActivityResult() - called when switching back to this activity
	 * 		in my code, requestCode can only be SELECT_TROPHY_ACTIVITY
	 * 
	 * @param requestCode - integer for knowing where the result is from
	 * @param resultCode - integer returned by the child activity
	 * @param data - String data result of the child activity
	 * @param extras - Bundle holding additional result information
	 */
	protected void onActivityResult(int requestCode, int resultCode, String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		
		if (requestCode == SELECT_TROPHY_ACTIVITY) {
			if (resultCode == RESULT_CANCELED) {
				return;
			}
			
			// get the Bundle info passed from the SelectTrophy Activity
			int imageIdToUse = extras.getInt(EditRoom.IMAGE_KEY);
			User.getCurrentUser(this).updateRoom(shelfPosBeingReplaced, imageIdToUse, this);
			//User.getCurrentUser(this).save(this);
			// reload the grid
			GridView g = (GridView) findViewById(R.id.myGrid);
			g.setAdapter(new ImageAdapterRoomView(this, User.getCurrentUser(this)));
			g.setSelection(shelfPosBeingReplaced);
			
			pd = ProgressDialog.show(this, "Contacting server", "Saving room", true, false);
			new SaveRoomThread().start();
			
			// display message to let the user know that the changes were saved
			//Toast.makeText(EditRoom.this, "change saved" , Toast.LENGTH_SHORT).show();
			
			
			return;
		}
		else {
			return;
		}
	}
	
	protected class SaveRoomThread extends Thread
	{
		public void run() {
			String response = User.getCurrentUser(EditRoom.this).save(EditRoom.this);
			pd.dismiss();
			
			if (response == null) {
				mHandler.post(new ShowDialogThread());
			}
		}
	}
	
	protected class ShowDialogThread extends Thread
	{
	   	public void run() {
	   		noConnectionDialog = new NoConnectionDialog(EditRoom.this);
			noConnectionDialog.show();
			noConnectionDialog.setTryAgainListener(new TryAgainListener());
			noConnectionDialog.setCancelListener(new CancelListener());
	   	}
	}
	   
	   
	protected class TryAgainListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
			pd = ProgressDialog.show(EditRoom.this, "Contacting server", "Saving room", true, false);
			new SaveRoomThread().start();
		}
	}
	
	protected class CancelListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
		}
	}
	
	// handles key presses
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {		
		super.onKeyDown(keyCode, event);
		
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK: {  // go back to EditRoomAvatarMain
			setResult(RESULT_OK);
			finish();
			return true;
		}
		}
		return false;
	}
	
	@Override
	protected void onPause() 
	{
		super.onPause();
		
    	
	}
    
    
    // makeView() - from Android View Gallery's Grid2.java file
    public View makeView() {
    	ImageView i = new ImageView(this);
        i.setBackgroundColor(0xFF000000);
        i.setScaleType(ImageView.ScaleType.FIT_CENTER);
        i.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
                LayoutParams.FILL_PARENT));
        return i;
    }
}