package net.porkbuns.my_story;

import net.porkbuns.my_story.api.User;
import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Camera;
import android.graphics.Matrix;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.KeyEvent;
import android.view.View;
import android.view.animation.AccelerateInterpolator;
import android.view.animation.Animation;
import android.view.animation.Transformation;

public class Home extends Activity {
	private AvatarLayout mAvatar;
	private ProgressDialog pd;
	private MyHandler msgHandler;
	private NoConnectionDialog noConnectionDialog;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.home);
		setTitle("My Story");
		Intent i;
		mAvatar = (AvatarLayout) this.findViewById(R.id.avatar_layout);
		msgHandler = new MyHandler();

		if (!User.hasAccount(this)) { // prompt user to create an account if
										// needed
			i = new Intent(this, CreateAccount.class);
			startSubActivity(i, 0);
		} else {
			pd = ProgressDialog.show(this, "Contacting server", "Loading user data", true, false);
			new LoadThread().start();
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		;
		
		switch (resultCode) {
		case RESULT_CANCELED:
		case RESULT_OK:
			startService(new Intent(this, ProximityChecker.class), null);
			View container = findViewById(R.id.widget0);
			mAvatar.setUser(User.getCurrentUser(Home.this));
			mAvatar.redraw();
			container.invalidate();
			break;
		case RESULT_FIRST_USER:
			finish();
			break;
		}
		;
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		final View container = findViewById(R.id.widget0);
		Intent i;
		Rotate3dAnimation rotation;
		switch (keyCode) {
		case KeyEvent.KEYCODE_DPAD_UP: // Quests
			i = new Intent(this, QuestTabs.class);
			rotation = new Rotate3dAnimation(true, 0, 90, container, 310.0f, true);
			break;
		case KeyEvent.KEYCODE_DPAD_RIGHT: // You
			i = new Intent(this, EditRoomAvatarMain.class);
			rotation = new Rotate3dAnimation(false, 0, 90, container, 310.0f, true);
			break;
		case KeyEvent.KEYCODE_DPAD_DOWN: // Quest Maps
			i = new Intent(this, QuestMapOverall.class);
			rotation = new Rotate3dAnimation(true, 0, -90, container, 310.0f, true);
			break;
		case KeyEvent.KEYCODE_DPAD_LEFT: // Friends!
			i = new Intent(this, Friends.class);
			rotation = new Rotate3dAnimation(false, 0, -90, container, 310.0f, true);
			break;
		default:
			return super.onKeyDown(keyCode, event);
		}
		// Rotate the home screen
		rotation.setDuration(200);
		rotation.setFillAfter(true);
		rotation.setInterpolator(new AccelerateInterpolator());
		rotation.setAnimationListener(new DisplayNextView(i));

		container.startAnimation(rotation);
		return true;
	}

	/**
	 * This class listens for the end of the first half of the animation. It
	 * then posts a new action that effectively swaps the views when the
	 * container is rotated 90 degrees and thus invisible.
	 */
	private final class DisplayNextView implements Animation.AnimationListener {
		private final Intent intent;

		private DisplayNextView(Intent intent) {
			this.intent = intent;
		}

		public void onAnimationStart() {
		}

		public void onAnimationEnd() {
			startSubActivity(intent, 0);
		}

		public void onAnimationRepeat() {
		}
	}

	protected class LoadThread extends Thread {
		public void run() {
			User currentUser = User.getCurrentUser(Home.this);
			if (currentUser != null) {
				mAvatar.setUser(currentUser);
				msgHandler.sendEmptyMessage(10);
			}
			else {
				msgHandler.sendEmptyMessage(11);
			}
		}
	}

	protected class MyHandler extends Handler {
		public void handleMessage(Message msg) {
			super.handleMessage(msg);
			if (msg.what == 10) {
				mAvatar.redraw();
				pd.dismiss();
				startService(new Intent(Home.this, ProximityChecker.class), null);
			}
			else if (msg.what == 11) {
				pd.dismiss();
				noConnectionDialog = new NoConnectionDialog(Home.this);
				noConnectionDialog.show();
				noConnectionDialog.setTryAgainListener(new TryAgainListener());
				noConnectionDialog.setCancelListener(new CancelListener());
			}
		}
	}
	
	protected class TryAgainListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
			pd = ProgressDialog.show(Home.this, "Contacting server", "Loading user data", true, false);
			new LoadThread().start();
		}
	}
	
	protected class CancelListener implements View.OnClickListener {
		public void onClick(View v) {
			finish();
		}
	}
	
	/**
	 * An animation that rotates the view on the Y axis between two specified
	 * angles. This animation also adds a translation on the Z axis (depth) to
	 * improve the effect.
	 */
	private class Rotate3dAnimation extends Animation {
		private final boolean mRotateX;
		private final float mFromDegrees;
		private final float mToDegrees;
		private final float mCenterX;
		private final float mCenterY;
		private final float mDepthZ;
		private final boolean mReverse;
		private Camera mCamera;

		public Rotate3dAnimation(boolean rotateX, float fromDegrees, float toDegrees, View v, float depth, boolean reverse) {
			this(rotateX, fromDegrees, toDegrees, v.getWidth() / 2.0f, v.getHeight() / 2.0f, depth, reverse);
		}

		/**
		 * Creates a new 3D rotation on the Y axis. The rotation is defined by its
		 * start angle and its end angle. Both angles are in degrees. The rotation
		 * is performed around a center point on the 2D space, definied by a pair of
		 * X and Y coordinates, called centerX and centerY. When the animation
		 * starts, a translation on the Z axis (depth) is performed. The length of
		 * the translation can be specified, as well as whether the translation
		 * should be reversed in time.
		 * 
		 * @param fromDegrees
		 *            the start angle of the 3D rotation
		 * @param toDegrees
		 *            the end angle of the 3D rotation
		 * @param centerX
		 *            the X center of the 3D rotation
		 * @param centerY
		 *            the Y center of the 3D rotation
		 * @param reverse
		 *            true if the translation should be reversed, false otherwise
		 */
		public Rotate3dAnimation(boolean rotateX, float fromDegrees, float toDegrees, float centerX, float centerY,
				float depthZ, boolean reverse) {
			mRotateX = rotateX;
			mFromDegrees = fromDegrees;
			mToDegrees = toDegrees;
			mCenterX = centerX;
			mCenterY = centerY;
			mDepthZ = depthZ;
			mReverse = reverse;
		}

		@Override
		public void initialize(int width, int height, int parentWidth, int parentHeight) {
			super.initialize(width, height, parentWidth, parentHeight);
			mCamera = new Camera();
		}

		@Override
		protected void applyTransformation(float interpolatedTime, Transformation t) {
			final float fromDegrees = mFromDegrees;
			float degrees = fromDegrees + ((mToDegrees - fromDegrees) * interpolatedTime);

			final float centerX = mCenterX;
			final float centerY = mCenterY;
			final Camera camera = mCamera;

			final Matrix matrix = t.getMatrix();

			camera.save();
			if (mReverse) {
				camera.translate(0.0f, 0.0f, mDepthZ * interpolatedTime);
			} else {
				camera.translate(0.0f, 0.0f, mDepthZ * (1.0f - interpolatedTime));
			}
			
			if (mRotateX) {
				camera.rotateX(degrees);
			} else {
				camera.rotateY(degrees);
			}
			
			camera.getMatrix(matrix);
			camera.restore();

			matrix.preTranslate(-centerX, -centerY);
			matrix.postTranslate(centerX, centerY);
		}
	}
}