package net.porkbuns.my_story;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.locks.ReentrantLock;

import net.porkbuns.my_story.api.Quest;
import net.porkbuns.my_story.api.User;
import android.app.Service;
import android.content.Intent;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.widget.Toast;

public class ProximityChecker extends Service
{
	private Handler mHandler = new Handler();
	public static ArrayList<Quest.Objective> nearObjectives;
	public static HashMap<Integer, QuestCompleter> pendingObjectives = new HashMap<Integer, QuestCompleter>();
	public static ReentrantLock lock = new ReentrantLock();
	
	 protected void onStart(int startId, Bundle arguments) {
		User currentUser = User.getCurrentUser(ProximityChecker.this);
		nearObjectives = currentUser.getActiveObj();
		new ObjectiveChecker().start();
	}
	public IBinder onBind(Intent intent) {
		 return null;
	}
	
	protected class ObjectiveChecker extends Thread
	{
		public void run() {
			while (true) {
				if (nearObjectives != null) {
					lock.lock();
					for (int i = 0; i < nearObjectives.size(); i++) {
						Quest.Objective o = nearObjectives.get(i);
						float dist = getDistance(o.getLatitude(), o.getLongitude());
						QuestCompleter qc;
						
						//if within 40 meters and haven't scheduled completion yet, then schedule it
						//if outside of 40 meters and commpletion was scheduled, cancel it
						if (dist <= 40 && !pendingObjectives.containsKey(new Integer(o.getId()))) {
							Quest q = Quest.get(o.getQuestId());
							qc = new QuestCompleter(o, q);
							pendingObjectives.put(new Integer(o.getId()), qc);
							qc.start();
						} else if (dist > 40 && pendingObjectives.containsKey(new Integer(o.getId()))) {
							qc = pendingObjectives.remove(new Integer(o.getId()));
							qc.stopRunning();
						}
					}
					lock.unlock();
				}
				try {
					sleep(500);
				}
				catch (Exception e) {
					
				}
			}
		}
		
		private int getDistance(double currentLat, double currentLong) {
			User currentUser = User.getCurrentUser(ProximityChecker.this);
			Location userLoc = new Location();
			userLoc.setLongitude(currentUser.getLongitude());
			userLoc.setLatitude(currentUser.getLatitude());
			
			Location objLoc = new Location();
			objLoc.setLatitude(currentLat);
			objLoc.setLongitude(currentLong);
			
			return (int)(userLoc.distanceTo(objLoc));
		}
	}
	
	public class QuestCompleter extends Thread {
		private Quest.Objective obj;
		private Quest quest;
		private boolean run = true;

		public QuestCompleter(Quest.Objective o, Quest q) {
			this.obj = o;
			this.quest = q;
		}

		public void run() {
			while (obj.getSeconds() > 0 && run) {
				try {
					sleep(1000);
					obj.decrementSeconds();
				}
				catch (Exception e) {
					
				}
			}
			
			if (run) {
				User currentUser = User.getCurrentUser(ProximityChecker.this);
				currentUser.completeObjective(obj, ProximityChecker.this);
				String text = "Objective completed for quest \"" + quest.getTitle() + "\"";
				
				if (quest.getStatus() == Quest.Status.COMPLETED) {
					text += "\nQuest is now complete.";
				}
				lock.lock();
				pendingObjectives.remove(new Integer(obj.getId()));
				nearObjectives.remove(obj);
				lock.unlock();
				mHandler.post(new RunThread(text));
			}
			else {
				obj.resetSeconds();
			}
		}
		
		public void stopRunning() {
			run = false;
		}
		
		protected class RunThread extends Thread
		{
			private String toastText;
			public RunThread(String output) {
				toastText = output;
			}
			public void run() {
				Toast t = Toast.makeText(ProximityChecker.this, toastText, 100);
				t.show();
			}
		}
	}
}

