package net.porkbuns.my_story;

import java.util.LinkedList;

import net.porkbuns.my_story.api.Item;
import net.porkbuns.my_story.api.Quest;
import net.porkbuns.my_story.api.User;
import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Menu;
import android.widget.ImageView;
import android.widget.TableLayout;
import android.widget.TableRow;
import android.widget.TextView;

public class QuestDetails extends Activity {
	 private Quest quest;
	 
	 public void onCreate(Bundle icicle) {
		 super.onCreate(icicle);
	     setContentView(R.layout.questdetails);
	     setTitle("My Story > Quests > Quest Details");
	     int questId = getIntent().getIntExtra("Id", 0);
	     quest = Quest.get(questId);
	     updateQuestView(); //draw the view for this quest
	 }
	 
	 public boolean onCreateOptionsMenu(Menu menu) {
	        super.onCreateOptionsMenu(menu);
	        //only show accept quest if the current quest is new
	        if (quest.getStatus() == Quest.Status.NEW) {
	        	menu.add(0,1, "Accept", R.drawable.menu_active);
	        	menu.add(0,2, "View Map of Objectives", R.drawable.menu_objectives);
	        }
	        else {
	        	menu.add(0,2, "View Map of Objectives", R.drawable.menu_objectives);
	        }
	        return true;
	 }
	 
	 public boolean onMenuItemSelected(int featureId, Menu.Item item) {
		 	switch(item.getId()) {
		 	case 1: //user selected accept quest
		 		User currentUser = User.getCurrentUser(this);
		 		
		 		currentUser.acceptQuest(quest, this);
		 		 
		 		//Update display of status
			    TextView status = (TextView) findViewById(R.id.quest_status);
			    status.setText("ACTIVE");
			    status.setTextColor(Color.argb(0xff, 0x99, 0xff, 0xff));
			    
			    //Get rid of accept quest instructions
			    ((TextView)findViewById(R.id.detail_instructions)).setText("View objectives map using the menu");
		 		
		 		break;
		 	case 2:
		 		Intent intent = new Intent(this, QuestMap.class);
		 		intent.putExtra("Id", quest.getId());
		 		startSubActivity(intent, 0);
		 		break;
		 	}
		 	
		 	return true;
	 }
	 
	//Hides the accept quest option if you accept the quest
    public boolean onPrepareOptionsMenu(Menu menu)
    {   
    	//hide accept quest if the quest is not new
    	if (quest.getStatus() != Quest.Status.NEW) {
	    	menu.setItemShown(1, false);
	    }
    	
    	return true;
    }
	 
	 //android.R.drawable.list_highlight_background_blue
	 //android.R.drawable.dialog_frame
	 protected void updateQuestView()
	 {
		 //Update display of the quest name
		 TextView name = (TextView) findViewById(R.id.quest_name);
	     name.setText(quest.getTitle());
	     name.setTextColor(Color.WHITE);
	     
	     //Update display of the quest description
	     ((TextView)findViewById(R.id.description_label)).setTextColor(Color.YELLOW);
	     
	     TextView description = (TextView) findViewById(R.id.description);
	     description.setText(quest.getDescription());
	     
	     //Update display of status
	     TextView status = (TextView) findViewById(R.id.quest_status);
	     Quest.Status statusType = quest.getStatus();
	     status.setText(statusType.toString());
	     
	     if (statusType == Quest.Status.ACTIVE) {
	    	 status.setTextColor(Color.argb(0xff, 0x99, 0xff, 0xff));
	    	 ((TextView)findViewById(R.id.detail_instructions)).setText("View objectives map using the menu");
	     }
	     else if (statusType == Quest.Status.COMPLETED) {
	    	 status.setTextColor(Color.GREEN);
	    	 ((TextView)findViewById(R.id.detail_instructions)).setText("View objectives map using the menu");
	     }
	     else if (statusType == Quest.Status.NEW) {
	    	 status.setTextColor(Color.YELLOW);
	    	 ((TextView)findViewById(R.id.detail_instructions)).setText("Accept quest or view objectives map using the menu");
	     }
	      
	     String completedObjectivesText = "", incompleteObjectivesText = "";
	     
	     LinkedList<Quest.Objective> objectives = quest.getObjectives();
	     int objSize = objectives.size();
	     //Go through all the objectives for this quest and see which ones are completed and which ones are not
	     for (int i = 0; i < objSize; i++) {
	    	 Quest.Objective obj = objectives.get(i);
	    	 
	    	 if (obj.getCompleted()) {
	    		 completedObjectivesText += obj.getDescription() + "\n";
	    	 }
	    	 else {
	    		 incompleteObjectivesText += obj.getDescription() + "\n";
	    	 }
	     }
	     
	     if (completedObjectivesText.length() == 0) {
	    	 completedObjectivesText = "None";
	     }
	     if (incompleteObjectivesText.length() == 0) {
	    	 incompleteObjectivesText = "None";
	     }
	     //Update display of completed objectives
	     ((TextView)findViewById(R.id.completed_objectives_label)).setTextColor(Color.YELLOW);
	     TextView completeObjectives = (TextView) findViewById(R.id.completed_objectives);
	     completeObjectives.setText(completedObjectivesText);
	     
	     //Update display of incomplete objectives
	     ((TextView)findViewById(R.id.incomplete_objectives_label)).setTextColor(Color.YELLOW);
	     TextView incompleteObjectives = (TextView) findViewById(R.id.incomplete_objectives);
	     incompleteObjectives.setText(incompleteObjectivesText);
	     
	     //Update display of rewards
	     ((TextView)findViewById(R.id.rewards_label)).setTextColor(Color.YELLOW);
	     TableLayout rewardTable = (TableLayout) findViewById(R.id.rewards_table);
	     
	     LinkedList<Item> rewards = quest.getRewards();
	     int rewardsSize = rewards.size();
	     //Adds 1 row to the TableLayout for each reward, showing the text and the picture side-by-side
	     for (int i = 0; i < rewardsSize; i++) {
	    	 TableRow row = new TableRow(this);
	    	 Item obj = rewards.get(i);
	    	 
	    	 TextView rewardData = new TextView(this);
	    	 String rewardText = "";
	    	 rewardText += obj.getName() + "\n";
	    	 rewardText += obj.getDescription() + "\n";
	    	 rewardData.setText(rewardText);
	    	 rewardData.setWidth(220);
	    	 
	    	 ImageView rewardImage = new ImageView(this);
	    	 rewardImage.setImageBitmap(obj.getBitmap(this));
	    	 
	    	 row.addView(rewardData);
	    	 row.addView(rewardImage);
	    	 rewardTable.addView(row, new TableLayout.LayoutParams());
	     }
	 }
}
