package net.porkbuns.my_story;

import java.text.DecimalFormat;
import java.util.LinkedList;

import net.porkbuns.my_story.api.Quest;
import net.porkbuns.my_story.api.User;
import android.app.Dialog;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.location.Location;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.ViewGroup;
import android.view.Menu.Item;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayController;
import com.google.android.maps.Point;
import com.google.android.maps.MapView.ReticleDrawMode;

public class QuestMap extends MapActivity {
	private MapView myMapView;
	private MapController mc;
	private OverlayController overlayController;
	private LinearLayout myLayout;
	private TextView distance;
	private double currentLat;
	private double currentLong;
	private Location userLoc;
	private LinkedList<Quest.Objective> objectiveList;
	private LinkedList<String> objectiveData = new LinkedList<String>();
	private int objPosition;
	private Dialog objectivePicker;
	
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.questmap);

		setTitle("My Story > Quests > Objective Map");

		myMapView = new MapView(this);
		myLayout = (LinearLayout) findViewById(R.id.map);
		myLayout.addView(myMapView, new LinearLayout.LayoutParams(
				ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));
		distance = (TextView) findViewById(R.id.distance);
		objPosition = 0;
		
		int questId = getIntent().getIntExtra("Id", 0);
		objectiveList = Quest.get(questId).getObjectives();
		
		loadDialog();
		
		mc = myMapView.getController();
		myMapView.setReticleDrawMode(ReticleDrawMode.DRAW_RETICLE_NEVER);
		overlayController = myMapView.createOverlayController();
		
		MyLocationOverlay myLocationOverlay = new MyLocationOverlay();
		overlayController.add(myLocationOverlay, true);
		
		currentLat = objectiveList.get(0).getLatitude(); 
		currentLong = objectiveList.get(0).getLongitude();
		mc.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
		mc.zoomTo(myMapView.getMaxZoomLevel());

		updateDistance();
		
		TextView legend = (TextView) findViewById(R.id.legend);
		legend.setText("Legend: red - completed, green - incomplete, blue - you");
	}

	private void loadDialog()
	{
		int objSize = objectiveList.size();
		String[] objNumbers = new String[objSize];
		for (int i = 0; i < objSize; i++) {
			Quest.Objective currObj = objectiveList.get(i);
			objNumbers[i] = currObj.getName();
			objectiveData.add("Description: " + currObj.getDescription() + "\n" +
							  "Status: " + (currObj.getCompleted() ? "Completed" : "Incomplete"));
		}
		ArrayAdapter<String> obj = new ArrayAdapter<String>(this,
												android.R.layout.simple_spinner_item, objNumbers);
		
		objectivePicker = new Dialog(this);
		objectivePicker.setContentView(R.layout.questmaplist);
		objectivePicker.setTitle("Select an objective");
		Spinner spin = (Spinner) objectivePicker.getWindow().findViewById (R.id.objective);
		spin.setAdapter(obj);
		spin.setOnItemSelectedListener(new ObjectiveSpinnerListener());
	}
	
	// handles key presses
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		super.onKeyDown(keyCode, event);

		switch (keyCode) {
		case KeyEvent.KEYCODE_DPAD_UP: { 
			currentLat += .0001;
			mc.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
			return true;
		}
		case KeyEvent.KEYCODE_DPAD_DOWN: { 
			currentLat -= .0001;
			mc.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
			return true;
		}
		case KeyEvent.KEYCODE_DPAD_LEFT: { 
			currentLong -= .0001;
			mc.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
			return true;
		}
		case KeyEvent.KEYCODE_DPAD_RIGHT: { 
			currentLong += .0001;
			mc.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
			return true;
		}
		}
		return false;
	}
	
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.add(0, 1, "Select objective");
		menu.add(0, 2, "Zoom");
		return true;
	}
	
	public boolean onMenuItemSelected(int featureId, Item item) {
		switch (item.getId()) {
		case 1:
			objectivePicker.show();
			break;
		case 2:
			float x = 100, y = 100;
			myMapView.displayZoomDialog(x, y);
			break;
		};
		
		return true;
	}
	
	private void updateDistance() {
		User currentUser = User.getCurrentUser(QuestMap.this);
		userLoc = new Location();
		userLoc.setLongitude(currentUser.getLongitude());
		userLoc.setLatitude(currentUser.getLatitude());
		
		Location objLoc = new Location();
		objLoc.setLongitude(currentLong);
		objLoc.setLatitude(currentLat);
		
		DecimalFormat dist = new DecimalFormat("0.0000"); 
		distance.setText("Distance from current location: " + dist.format(userLoc.distanceTo(objLoc)) + "m");
	}
	
	protected class ObjectiveSpinnerListener implements AdapterView.OnItemSelectedListener {
		@SuppressWarnings("unchecked")
		public void onItemSelected(AdapterView parent, View v, int position, long id) {
			TextView text = (TextView) objectivePicker.getWindow().findViewById (R.id.objective_data);
			text.setText(objectiveData.get(position));
			
			Quest.Objective o = objectiveList.get(position);
    		objPosition = position;
    		currentLat = o.getLatitude();
			currentLong = o.getLongitude();
			mc.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
			updateDistance();
		}
		
		public void onNothingSelected(AdapterView parent) {
			
		}
	}

	protected class MyLocationOverlay extends Overlay {
		@Override
		public void draw(Canvas canvas, PixelCalculator calculator, boolean shadow) {
			super.draw(canvas, calculator, shadow);
			int objSize = objectiveList.size();
			for (int i = 0; i < objSize; i++) {
				Quest.Objective currentObj = objectiveList.get(i);
				double originalLat = currentObj.getLatitude(), originalLong = currentObj.getLongitude();
				int[] myCoord = new int[2], objCoord = new int[2];
				drawDot(false, myCoord, canvas, calculator, shadow, userLoc.getLatitude(), 
						userLoc.getLongitude(), Color.BLUE);
				if (currentObj != null) {
					if (currentObj.getCompleted()) {
						drawDot(i == objPosition, objCoord, canvas, calculator, shadow, 
								originalLat, originalLong, Color.RED);
					}
					else {
						drawDot(i == objPosition, objCoord, canvas, calculator, shadow, 
								originalLat, originalLong, Color.GREEN);
					}
				}
				if (i == objPosition) {
					Paint p = new Paint();
					p.setStrokeWidth(2);
					p.setAlpha(50);
					canvas.drawLine(objCoord[0], objCoord[1], myCoord[0], myCoord[1], p);
				}
			}
		}
		
		protected void drawDot(boolean selected, int[] friendScreenCoords, Canvas canvas, 
				PixelCalculator calculator, boolean shadow, double lat, double lng, int color) {
			Paint paint = new Paint();
			Point point = new Point((int) (lat * 1E6), (int) (lng * 1E6));
			
			// Converts lat/lng-Point to coordinates on the screen.
			calculator.getPointXY(point, friendScreenCoords);
			if (Math.abs(friendScreenCoords[0]) < 2000
					&& Math.abs(friendScreenCoords[1]) < 2000) {

				paint.setColor(color);
				paint.setStrokeWidth(1);
				canvas.drawCircle(friendScreenCoords[0], friendScreenCoords[1], 4, paint);
				
				paint.setColor(Color.BLACK);

				// With a black stroke around the oval we drew before.
				paint.setColor(Color.BLACK);
				paint.setStyle(Style.STROKE);
				canvas.drawCircle(friendScreenCoords[0], friendScreenCoords[1], 4, paint);
				if (selected) {
					canvas.drawCircle(friendScreenCoords[0], friendScreenCoords[1], 7, paint);
				}
			}
		}
	}
	

}