package net.porkbuns.my_story;


import java.util.LinkedList;
import java.util.TreeMap;

import net.porkbuns.my_story.api.Quest;
import net.porkbuns.my_story.api.User;
import android.app.Dialog;
import android.content.DialogInterface;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.location.Location;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.view.ViewGroup;
import android.view.Menu.Item;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayController;
import com.google.android.maps.Point;
import com.google.android.maps.MapView.ReticleDrawMode;

public class QuestMapOverall extends MapActivity {

	// ===========================================================
	// Fields
	// ===========================================================

	/*
	 * Stuff we need to save as fields, because we want to use multiple of them
	 * in different methods.
	 */
	protected boolean doUpdates = true;
	protected MapView myMapView = null;
	protected MapController myMapController = null;
	protected OverlayController myOverlayController = null;
	private Handler mHandler = new Handler();

	private TreeMap <Integer, Quest.Objective> sortedDistances;
	private LinkedList<Quest.Objective> spinnerObjectives;
	private Dialog objectivePicker;
	public static Spinner objectiveSpinner;
	private UpdateQuestText questUpdater;

	protected class MyLocationOverlay extends Overlay {
		@Override
		public boolean draw(Canvas canvas, PixelCalculator calculator, boolean shadow, long when) {
			super.draw(canvas, calculator, shadow);
			double lat, lng;
			float radius;
			User currentUser = User.getCurrentUser(QuestMapOverall.this);
			
			//only draw dots for objectives that are not complete
			if (ProximityChecker.nearObjectives != null) {
				int objSize = ProximityChecker.nearObjectives.size();
				for (int i = 0; i < objSize; i++) {
					Quest.Objective o = ProximityChecker.nearObjectives.get(i);
					lat = o.getLatitude();
					lng = o.getLongitude();
					radius = (float)((o.getSeconds() * 4.0)/o.getOriginalSeconds());
					
					if (!ProximityChecker.pendingObjectives.containsKey(new Integer(o.getId()))) {
						drawDot(canvas, calculator, shadow, lat, lng, Color.GREEN, radius);
					}
					else {
						drawDot(canvas, calculator, shadow, lat, lng, Color.YELLOW, radius);
					}
				}
			}
			drawDot(canvas, calculator, shadow, currentUser.getLatitude(), currentUser.getLongitude(), Color.BLUE, 4);
			return true;
		}
		
		protected void drawDot(Canvas canvas, PixelCalculator calculator, boolean shadow, double lat, double lng, int color, float radius) {
			Paint paint = new Paint();
			paint.setTextSize(14);
			int[] friendScreenCoords = new int[2];
			Point point = new Point((int) (lat * 1E6), (int) (lng * 1E6));
			
			// Converts lat/lng-Point to coordinates on the screen.
			calculator.getPointXY(point, friendScreenCoords);
			if (Math.abs(friendScreenCoords[0]) < 2000
					&& Math.abs(friendScreenCoords[1]) < 2000) {
				
				
				paint.setColor(color);
				paint.setStrokeWidth(1);
				canvas.drawCircle(friendScreenCoords[0], friendScreenCoords[1], radius, paint);

				// With a black stroke around the oval we drew before.
				paint.setColor(Color.BLACK);
				paint.setStyle(Style.STROKE);
				canvas.drawCircle(friendScreenCoords[0], friendScreenCoords[1], radius, paint);
			}
		}
	}

	// ===========================================================
	// """Constructor""" (or the Entry-Point of this class)
	// ===========================================================
	@Override
	protected void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		
		/* Create a new MapView and show it */
		this.myMapView = new MapView(this);
		this.setContentView(R.layout.questmapoverall);
		LinearLayout myLayout = (LinearLayout) findViewById(R.id.map);
		myLayout.addView(myMapView, new LinearLayout.LayoutParams(
					ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));
		
		this.myMapController = this.myMapView.getController();
		setTitle("My Story > Map Overview");
		
		User currentUser = User.getCurrentUser(this);
		this.myMapController.zoomTo(myMapView.getMaxZoomLevel()); 
		this.myMapController.animateTo(new Point((int) (currentUser.getLatitude() * 1E6),
												 (int) (currentUser.getLongitude() * 1E6)));

		this.myOverlayController = this.myMapView.createOverlayController();
		MyLocationOverlay myLocationOverlay = new MyLocationOverlay();
		this.myOverlayController.add(myLocationOverlay, true);
		
		myMapView.setReticleDrawMode(ReticleDrawMode.DRAW_RETICLE_NEVER);
	}
	
	private void loadDialog()
	{
		ProximityChecker.lock.lock();
		int objSize = ProximityChecker.nearObjectives.size();
		
		sortedDistances = new TreeMap<Integer, Quest.Objective>();
		spinnerObjectives = new LinkedList<Quest.Objective>();
		for (int i = 0; i < objSize; i++) {
			Quest.Objective currObj = ProximityChecker.nearObjectives.get(i);
			sortedDistances.put(new Integer(getDistance(currObj.getLatitude(), currObj.getLongitude())), currObj);
			spinnerObjectives.add(currObj);
		}
		ProximityChecker.lock.unlock();
		
		Integer[] distances = sortedDistances.keySet().toArray(new Integer[objSize]);
		String[] distanceStrings = new String[objSize];
		
		for (int i = 0; i < objSize; i++) {
			distanceStrings[i] = "Distance (m): " + distances[i];
		}
		
		ArrayAdapter<String> obj = new ArrayAdapter<String>(this,
												android.R.layout.simple_spinner_item, distanceStrings);
		
		objectiveSpinner.setAdapter(obj);
		objectiveSpinner.setOnItemSelectedListener(new ObjectiveSpinnerListener());
		
		if (objSize == 0) {
			TextView text = (TextView) objectivePicker.getWindow().findViewById (R.id.objective_data);
			text.setText("");
		}
		
		questUpdater = new UpdateQuestText();
		questUpdater.start();
	}
	
	
	private int getDistance(double currentLat, double currentLong) {
		User currentUser = User.getCurrentUser(this);
		Location userLoc = new Location();
		userLoc.setLongitude(currentUser.getLongitude());
		userLoc.setLatitude(currentUser.getLatitude());
		
		Location objLoc = new Location();
		objLoc.setLatitude(currentLat);
		objLoc.setLongitude(currentLong);
		
		return (int)(userLoc.distanceTo(objLoc));
	}
	
	protected class ObjectiveSpinnerListener implements AdapterView.OnItemSelectedListener {
		@SuppressWarnings("unchecked")
		public void onItemSelected(AdapterView parent, View v, int position, long id) {
			if (spinnerObjectives != null && position != -1) {
				Quest.Objective o = spinnerObjectives.get(position);
				
				double currentLat = o.getLatitude(), currentLong = o.getLongitude();
	    		myMapController.animateTo(new Point((int) (currentLat * 1000000), (int) (currentLong * 1000000)));
			}
		}
		
		public void onNothingSelected(AdapterView parent) {
			
		}
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		super.onKeyDown(keyCode, event);
		User currentUser = User.getCurrentUser(this);
		if (keyCode == KeyEvent.KEYCODE_DPAD_LEFT) {
			currentUser.changeLongitude(-.0001);
			adjustMap();
		} else if (keyCode == KeyEvent.KEYCODE_DPAD_RIGHT) {
			currentUser.changeLongitude(.0001);
			adjustMap();
		} else if (keyCode == KeyEvent.KEYCODE_DPAD_UP) {
			currentUser.changeLatitude(.0001);
			adjustMap();
		} else if (keyCode == KeyEvent.KEYCODE_DPAD_DOWN) {
			currentUser.changeLatitude(-.0001);
			adjustMap();
		} 
		
		return false;
	}
	
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.add(0, 1, "View objective status", R.drawable.menu_objectives);
		menu.add(0, 2, "Zoom", R.drawable.menu_search);
		return true;
	}
	
	public boolean onMenuItemSelected(int featureId, Item item) {
		switch (item.getId()) {
		case 1:
			objectivePicker = new Dialog(this);
			objectivePicker.setContentView(R.layout.questmaplist);
			objectivePicker.setTitle("Select an objective");
			objectivePicker.setCancelListener(new CancelListener());
			objectiveSpinner = (Spinner) objectivePicker.getWindow().findViewById (R.id.objective);
			loadDialog();
			objectivePicker.show();
			break;
		case 2:
			float x = 100, y = 100;
			myMapView.displayZoomDialog(x, y);
			break;
		};
		
		return true;
	}
	
	private void adjustMap()
	{
		User currentUser = User.getCurrentUser(this);
		double currentLatitude = currentUser.getLatitude();
		double currentLongitude = currentUser.getLongitude();
		
		this.myMapController.centerMapTo(new Point((int) (currentLatitude * 1E6),
				(int) (currentLongitude * 1E6)), true);
	}
	
	public class CancelListener implements DialogInterface.OnCancelListener
	{
		public void onCancel(DialogInterface dialog) {
			questUpdater.stopRunning();
		}
	}

	public class UpdateQuestText extends Thread {
		boolean shouldRun;
		public UpdateQuestText() {
			shouldRun = true;
		}
		public void run() {
			while (shouldRun) {
				if (spinnerObjectives != null && objectiveSpinner.getSelectedItemPosition() != -1) {
					Quest.Objective o = spinnerObjectives.get(objectiveSpinner.getSelectedItemPosition());
					Quest quest = Quest.get(o.getQuestId());
					mHandler.post(new StatusThread("Quest Title: " + quest.getTitle() + "\n" +
							 "Quest Objective: " + o.getName() + " (" + o.getDescription() + ")\n" +
							 "# of objectives remaining for quest: " + quest.getNumIncompleteObj() + "\n" +
							 "Time Remaining: " + o.getSeconds() + " second(s)\n"));
					
					try {
						sleep(250);
					}
					catch(Exception e) {
						
					}
				}
			}
		}
		public void stopRunning() {
			shouldRun = false;
		}
	}
	
	public class StatusThread extends Thread {
		private String statusText;
		public StatusThread(String output) {
			statusText = output;
		}
		public void run() {
			TextView text = (TextView) objectivePicker.getWindow().findViewById (R.id.objective_data);
			text.setText(statusText);
		}
	}
}
