package net.porkbuns.my_story;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;

import net.porkbuns.my_story.api.Quest;
import android.app.ActivityGroup;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;

public class QuestSearch extends ActivityGroup  {
	private ArrayList<HashMap<String,String>> questList;
	private ListView search_results_list;
	private LinearLayout ll;
	private View search_box_view;
	private View search_results_view;
	private TextView query;
	private TextView num_results_view;
	private LinkedList<Quest> currentSearchResults;
	private ProgressDialog pd;
	private Handler handler;
	private NoConnectionDialog noConnectionDialog;
	
	public void onCreate(Bundle icicle) {
		 super.onCreate(icicle);
	     setContentView(R.layout.questsearch);
	     questList = new ArrayList<HashMap<String,String>>();
	     currentSearchResults = new LinkedList<Quest>();
	     
	     Button searchButton = (Button) findViewById(R.id.search_button);
	     searchButton.setOnClickListener(new SearchButtonListener());
	     
	     //Store the views which correspond to the search_box and the search_results
	     search_results_list = (ListView) findViewById(R.id.search_list);
	     query = (TextView) findViewById(R.id.search_entry);
	     num_results_view = (TextView) findViewById(R.id.num_results);
	     
	     ll = (LinearLayout) findViewById(R.id.search);
	     search_box_view = ll.getChildAt(0);
	     search_results_view = ll.getChildAt(1);
	     ll.removeView(search_results_view);
	     
	     handler = new Handler();
	 }
	
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK: // Quests
			if (ll.indexOfChild(search_results_view) != -1) {
				questList = new ArrayList<HashMap<String,String>>();
				updateListAdapter(search_results_list); //clear the results of the search
				ll.removeView(search_results_view);
				if (ll.indexOfChild(search_box_view) == -1) {
					ll.addView(search_box_view);
				}
				ll.requestFocus();
			}
			else {
				return super.onKeyDown(keyCode, event);
			}
			break;
		default:
			return super.onKeyDown(keyCode, event);
		}
		return true;
	}
	 
    protected class SearchButtonListener implements View.OnClickListener 
    {
    	public void onClick(View view) {
    		//Remove the search box view and replace it with the search results view
    		ll.removeView(search_box_view);
    		if (ll.indexOfChild(search_results_view) == -1) {
    			ll.addView(search_results_view);
    		}
    		ll.requestFocus();
    		
    		pd = ProgressDialog.show(QuestSearch.this, "Contacting server", "Searching for quests", true, false);
    		
    		new SearchThread().start();
    	}
    }
    
    protected class SearchThread extends Thread {
    	public void run() {
    		questList = new ArrayList<HashMap<String,String>>();
    		search_results_list.setOnItemClickListener(new QuestItemListener(QuestSearch.this));
    		String queryText = query.getText().toString();    	
    		currentSearchResults = Quest.search(queryText);
    		pd.dismiss();
    		if (currentSearchResults != null) {
	    		int numResults = currentSearchResults.size();
				
	 			for (int j = 0; j < numResults; j++) {
					Quest currentQuest = currentSearchResults.get(j);
					addListItem(currentQuest.getTitle(), currentQuest.getDescription());
				}
				
	 			handler.post(new UpdateUIThread(numResults));
	    		setResult(0);
    		}
    		else {
    			handler.post(new ShowDialogThread());
			}
    	}
    }
    
    protected class ShowDialogThread extends Thread
    {
    	public void run() {
    		noConnectionDialog = new NoConnectionDialog(QuestSearch.this);
			noConnectionDialog.show();
			noConnectionDialog.setTryAgainListener(new TryAgainListener());
			noConnectionDialog.setCancelListener(new CancelListener());
    	}
    }
    
    
    protected class TryAgainListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
			pd = ProgressDialog.show(QuestSearch.this, "Contacting server", "Searching for quests", true, false);
			new SearchThread().start();
		}
	}
	
	protected class CancelListener implements View.OnClickListener {
		public void onClick(View v) {
			noConnectionDialog.dismiss();
			if (ll.indexOfChild(search_results_view) != -1) {
				questList = new ArrayList<HashMap<String,String>>();
				updateListAdapter(search_results_list); //clear the results of the search
				ll.removeView(search_results_view);
				if (ll.indexOfChild(search_box_view) == -1) {
					ll.addView(search_box_view);
				}
				ll.requestFocus();
			}
		}
	}
    
    protected class UpdateUIThread extends Thread {
    	private int numResults;
    	public UpdateUIThread(int numResults) {
    		this.numResults = numResults;
    	}
    	public void run() {
			updateListAdapter(search_results_list);
    		num_results_view.setText("Number of results: " + numResults);
		}
    }
    
    protected class QuestItemListener implements AdapterView.OnItemClickListener {
    	private Context contxt;
    	public QuestItemListener(Context c) {
    		contxt = c;
    	}
    	
    	//If a quest in the list is clicked, load its details
    	@SuppressWarnings("unchecked")
		public void onItemClick(AdapterView parent, View v, int position, long id) {
    		Intent details = new Intent(contxt, QuestDetails.class);
    		details.putExtra("Id", currentSearchResults.get(position).getId());
        	startSubActivity(details, 0);
    	}
    }
    
    //Add this quest to the questList
    protected void addListItem(String questName, String questTitle)
    {
    	HashMap<String,String> item = new HashMap<String,String>();
  	  	item.put("Name", questName );
  	  	item.put("Summary", questTitle);
  	  	questList.add( item );
    }
    
    //Load the data for the listview
    protected void updateListAdapter(ListView v)
    {
    	 SimpleAdapter notes = new SimpleAdapter( 
    			 	this, 
	  				questList,
	  				R.layout.questlisting,
	  				new String[] { "Name", "Summary" },
	                new int[] { R.id.text1, R.id.text2 } );
	        
		 v.setAdapter(notes);
    }
}