package net.porkbuns.my_story;

import android.app.ListActivity;
import android.os.Bundle;
import android.widget.TabHost;
import android.widget.TabHost.TabSpec;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.view.Menu;
import android.view.View;
import android.view.Menu.Item;
import android.content.Intent;
import android.app.LocalActivityManager;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;

import net.porkbuns.my_story.api.Quest;
import net.porkbuns.my_story.api.User;

import android.widget.Toast;

public class QuestTabs extends ListActivity {
	private ArrayList<HashMap<String, String>> questList;
	private TabHost tabs;
	private ArrayList<Quest> currentTabQuestList;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.quests);

		tabs = (TabHost) this.findViewById(R.id.tabhost);
		// more weirdness due to an Android bug :(
		tabs.setup(new LocalActivityManager(this, true));

		// create the active tab
		TabSpec tempTab = tabs.newTabSpec("Active");
		tempTab.setIndicator(new TabView(this, "Active", R.drawable.quest_icon_active));
		tempTab.setContent(R.id.list_layout);
		tabs.addTab(tempTab);

		// create the completed tab
		tempTab = tabs.newTabSpec("Completed");
		tempTab.setIndicator(new TabView(this, "Completed", R.drawable.quest_icon_complete));
		tempTab.setContent(R.id.list_layout);
		tabs.addTab(tempTab);

		// create the find new tab (it calls the Quest Search class)
		tempTab = tabs.newTabSpec("Find New");
		tempTab.setIndicator(new TabView(this, "Find New", R.drawable.quest_icon_new));
		tempTab.setContent(new Intent(this, QuestSearch.class));
		tabs.addTab(tempTab);

		tabs.setOnTabChangedListener(new QuestTabListener());
		questList = new ArrayList<HashMap<String, String>>();

		tabs.setCurrentTab(0); // start out on the current tab
	}

	// if an item in the list view is clicked, bring up its details
	protected void onListItemClick(ListView l, View v, int position, long id) {
		Intent details = new Intent(this, QuestDetails.class);
		details.putExtra("Id", currentTabQuestList.get(getSelectedItemPosition()).getId());
		startSubActivity(details, 0);
	}

	// used for demoing purposes so we can fake complete a quest
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		/*if (tabs.getCurrentTab() != 2) {
			menu.add(0, 1, "Complete quest", R.drawable.menu_active);
			menu.add(0, 2, "Clear tables", R.drawable.menu_clear);
		}*/
		return true;
	}

	// hardcoded to complete either quest 1 or 2, depending on which one is
	// currently selected
	public boolean onMenuItemSelected(int featureId, Item item) {
		QuestDBAdapter mAdapter;
		switch (item.getId()) {
		case 1:
			User currentUser = User.getCurrentUser(this);

			Quest currentQuest = currentTabQuestList.get(getSelectedItemPosition());
			if (currentQuest.getStatus() != Quest.Status.COMPLETED) {
				LinkedList<Quest.Objective> objectives = currentQuest.getObjectives();
				int objectivesSize = objectives.size();

				for (int i = 0; i < objectivesSize; i++) {
					currentUser.completeObjective(objectives.get(i), this);
				}

				// pop up notification that quest was completed
				Toast t = Toast.makeText(this, "Quest: " + currentQuest.getTitle() + " completed",
						100);
				t.show();

				// reload tags based on which quest got completed
				if (tabs.getCurrentTabTag() == null) {
					loadTab(Quest.Status.ACTIVE);
				} else if (tabs.getCurrentTabTag().equals("Completed")) {
					loadTab(Quest.Status.COMPLETED);
				}
			}
			break;
		case 2:
			mAdapter = new QuestDBAdapter(this);
			mAdapter.open();
			mAdapter.deleteAllQuests();
			mAdapter.deleteAllObjectives();
			mAdapter.close();
			break;
		}
		return true;
	}

	// Loads in the quest data for the given tab from the database
	protected void loadTab(Quest.Status status) {
		questList = new ArrayList<HashMap<String, String>>();
		// currentTabQuestList = new LinkedList<Quest>();
		User currentUser = User.getCurrentUser(this);

		if (status == Quest.Status.COMPLETED) {
			currentTabQuestList = currentUser.getCompletedQuests();
		} else {
			currentTabQuestList = currentUser.getActiveQuests();
		}

		int numQuests = currentTabQuestList.size();
		for (int i = 0; i < numQuests; i++) {
			Quest currentQuest = currentTabQuestList.get(i);
			addListItem(currentQuest.getTitle(), currentQuest.getDescription());
		}

		/*
		 * QuestDBAdapter mAdapter = new QuestDBAdapter(this); mAdapter.open();
		 * Cursor cur = mAdapter.fetchAllQuests((status ==
		 * Quest.Status.COMPLETED)); int questId;
		 * 
		 * if (cur != null) { do { questId = cur.getInt(mAdapter.INDEX_QUESTID);
		 * Quest currentQuest = Quest.get(questId);
		 * currentTabQuestList.add(currentQuest); } while(cur.next());
		 * cur.close(); } mAdapter.close();
		 */
		
		if (numQuests > 0) {
			setSelection(0);
		}
		updateListAdapter(getListView());

	}

	// add a quest name/description to the questList
	protected void addListItem(String questName, String questTitle) {
		HashMap<String, String> item = new HashMap<String, String>();
		item.put("Name", questName);
		item.put("Summary", questTitle);
		questList.add(item);
	}

	// Setup the data for the listview
	protected void updateListAdapter(ListView lv) {
		SimpleAdapter notes = new SimpleAdapter(this, questList, R.layout.questlisting,
				new String[] { "Name", "Summary" }, new int[] { R.id.text1, R.id.text2 });

		lv.setAdapter(notes);
	}

	// Loads tabs and title names according to which tab you're on
	protected class QuestTabListener implements TabHost.OnTabChangeListener {
		public void onTabChanged(String tabId) {
			if (tabId == null) { // corresponds to 1st tab (bug in Android)
				loadTab(Quest.Status.ACTIVE);
				setTitle("My Story > Quests > Active");
			} else if (tabId.compareTo("Completed") == 0) {
				loadTab(Quest.Status.COMPLETED);
				setTitle("My Story > Quests > Completed");
			} else if (tabId.compareTo("Find New") == 0) {
				setTitle("My Story > Quests > Find New");
			}
		}
	}
}