package net.porkbuns.my_story;

import java.util.ArrayList;

import net.porkbuns.my_story.api.Item;
import net.porkbuns.my_story.api.User;
import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.BaseAdapter;
import android.widget.Gallery;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.Gallery.LayoutParams;

public class SelectTrophy extends Activity {
	// the position on the shelf to replace
	private int mShelfPosToReplace;
	private ArrayList<Item> trophies;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.select_trophy);
        setTitle("My Story > You > Edit Room");
        
        User currentUser = User.getCurrentUser(this);

		trophies = currentUser.getTrophies();
        Bundle extras = getIntent().getExtras();
        mShelfPosToReplace = extras.getInt(EditRoom.BITMAP_POS);
        
        GridView g = (GridView) findViewById(R.id.myGrid_select_items);
        g.setAdapter(new ImageAdapter(this));
        GridView g_display = (GridView) findViewById(R.id.myGrid);
        g_display.setAdapter(new ImageAdapterModifiedRoomView(this));
        
        // gray out the room display and make the room view un-selectable
        g_display.setEnabled(false);
        g_display.setFocusable(false);
        
        // set "clicks" on images to return to EditRoom
		g.setOnItemClickListener(new OnItemClickListener() {
			@SuppressWarnings("unchecked")
			public void onItemClick(AdapterView parent, View v, int position, long id) {
				Bundle bundle = new Bundle();
				
				// set the fields of the bundle for the ImageBrowser
				if (position != trophies.size()) {
					bundle.putInt(EditRoom.IMAGE_KEY,trophies.get(position).getId());
				}
				else {
					bundle.putInt(EditRoom.IMAGE_KEY,-1);
				}
				setResult(RESULT_OK, "", bundle);
				finish();
			}
		});
		
		Item roomItem = currentUser.getRoomItem(mShelfPosToReplace);
		if (roomItem != null) {
			g.setSelection(trophies.indexOf(roomItem));
		}
		else {
			g.setSelection(trophies.size() + 1);
		}
    }
    
    // handles key presses
	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {		
		super.onKeyDown(keyCode, event);
		
		switch (keyCode) {
		case KeyEvent.KEYCODE_BACK: {  // go back to the EditRoom
			Bundle bundle = new Bundle();
			
			// set the fields of the bundle for the ImageBrowser
			bundle.putString(EditRoom.IMAGE_KEY, "no_changes");
			bundle.putString(EditRoom.REPLACE_POS_KEY, "no_changes");
			setResult(RESULT_CANCELED, "", bundle);
			finish();
			return true;
		}
		}
		return false;
	}
    
    // makeView() - from Android View Gallery's Grid2.java file
    public View makeView() {
    	ImageView i = new ImageView(this);
        i.setBackgroundColor(0xFF000000);
        i.setScaleType(ImageView.ScaleType.FIT_CENTER);
        i.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT,
                LayoutParams.FILL_PARENT));
        return i;
    }
    
    // ImageAdapter class - slightly modified from Android View Gallery's Grid2.java file
    public class ImageAdapter extends BaseAdapter {
        public ImageAdapter(Context c) {
            mContext = c;
        }

        public int getCount() {
            return trophies.size() + 1;
        }

        public Object getItem(int position) {
            return position;
        }

        public long getItemId(int position) {
            return position;
        }

        public View getView(int position, View convertView, ViewGroup parent) {
            ImageView i = new ImageView(mContext);
            if (position != trophies.size()) {
            	i.setImageDrawable(trophies.get(position).getDrawable(mContext));
            }
            else {
            	i.setBackground(R.drawable.trophy_frame);
            }
            //i.setImageBitmap(mImages[position]);
            
            i.setLayoutParams(new Gallery.LayoutParams(40, 50));
            i.setAdjustViewBounds(false);
            i.setScaleType(ImageView.ScaleType.FIT_CENTER);
            return i;
        }

        private Context mContext;
    }
    
 // ImageAdapter class - slightly modified from Android View Gallery's Grid2.java file
    public class ImageAdapterModifiedRoomView extends BaseAdapter {
        public ImageAdapterModifiedRoomView(Context c) {
            mContext = c;
        }

        public int getCount() {
            return User.ROOMSIZE;
        }

        public Object getItem(int position) {
            return position;
        }

        public long getItemId(int position) {
            return position;
        }

        public View getView(int position, View convertView, ViewGroup parent) {
            ImageView i = new ImageView(mContext);
            User currentUser = User.getCurrentUser(mContext);
            Item roomItem = currentUser.getRoomItem(position); 
            if (roomItem != null) {
            	i.setImageDrawable(roomItem.getDrawable(mContext));
            }
            
            i.setLayoutParams(new Gallery.LayoutParams(60, 75));
            i.setAdjustViewBounds(false);
            i.setScaleType(ImageView.ScaleType.FIT_CENTER);
            if (position == mShelfPosToReplace) {
            	i.setBackground(android.R.drawable.button_background_pressed_yellow);
            }
            else {
            	i.setBackground(R.drawable.trophy_frame);
            }
            return i;
        }

		private Context mContext;
    }
}