package net.porkbuns.my_story.api;

import java.util.ArrayList;
import java.util.HashMap;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;

public class Item {
	public static enum Type {
		head, shirt, pants, trophy
	};

	private String description;
	private int id;
	private String imagePath;
	private Type what;
	private String name;
	private Bitmap image;
	private BitmapDrawable imageDrawable;
	
	private static HashMap<Integer, Item> cache = new HashMap<Integer, Item>();

	public static Item get(int id) {
		if (id < 0) {
			return null;
		}
		if (cache.containsKey(new Integer(id))) {
			return cache.get(new Integer(id));
		}
		else {
			Document d = ServerUtil.fetchXml("/items/" + id + ".xml");
			if (d != null) {
				return parseXml(d);
			}
			else {
				return null;
			}
		}
	}

	public static ArrayList<Item> getAll() {
		ArrayList<Item> result = new ArrayList<Item>();

		Document d = ServerUtil.fetchXml("/items.xml");
		if (d != null) {
			NodeList nodes = d.getChildNodes();
			int listLength = nodes.getLength();
			for (int i = 0; i < listLength; i++) {
				result.add(parseXml(nodes.item(i)));
			}
			return result;
		}
		else {
			return null;
		}
	}

	static Item parseXml(Node e) {
		String name = "", description = "", image_path = "", what = "";
		int id = 0;

		NodeList nl = e.getChildNodes();
		Node n;
		int listLength = nl.getLength();
		
		for (int idx = 0; idx < listLength; idx++) {
			n = nl.item(idx);
			if (n.getFirstChild() == null)
				continue;
			String text = n.getFirstChild().getNodeValue();
			String nodeName = n.getNodeName();
			if (nodeName.equals("item")) {
				return Item.parseXml(n);
			}
			else if (nodeName.equals("id")) {
				id = Integer.parseInt(text, 10);
				if (Item.cache.containsKey(new Integer(id))) {
					return Item.cache.get(new Integer(id));
				}
			} else if (nodeName.equals("description")) {
				description = text;
			} else if (nodeName.equals("image")) {
				image_path = text;
			} else if (nodeName.equals("name")) {
				name = text;
			} else if (nodeName.equals("what")) {
				what = text;
			}
		}
		
		Item result = new Item(name, description, id, image_path, what); 
		Item.cache.put(new Integer(id), result);
		return result;
	}

	public static ArrayList<Item> getAll(Type type) {
		ArrayList<Item> result = new ArrayList<Item>();

		HashMap<String, String> params = new HashMap<String, String>();
		params.put("what", type.name());
		Document d = ServerUtil.fetchXml(ServerUtil.RequestType.GET, "/items.xml", params);
		if (d != null) {
			NodeList nodes = d.getElementsByTagName("item");
			int listLength = nodes.getLength();
			for (int i = 0; i < listLength; i++) {
				result.add(parseXml(nodes.item(i)));
			}
			return result;
		}
		else {
			return null;
		}
	}

	public static ArrayList<Item> getAll(boolean male, Type type) {
		ArrayList<Item> result = new ArrayList<Item>();
		HashMap<String, String> params = new HashMap<String, String>();
		params.put("what", type.name());
		params.put("male", male ? "true" : "false");
		String url = "/items.xml";
		Document d = ServerUtil.fetchXml(ServerUtil.RequestType.GET, url, params);
		if (d != null) {
			NodeList nodes = d.getElementsByTagName("item");
			int listLength = nodes.getLength();
			for (int i = 0; i < listLength; i++) {
				result.add(parseXml(nodes.item(i)));
			}
			return result;
		}
		else {
			return null;
		}
	}

	private Item(String name, String description, int id, String image_path,
			String what) {
		this(name, description, id, image_path, Type.valueOf(what));
	}

	private Item(String name, String description, int id, String image_path,
			Type what) {
		this.name = name;
		this.description = description;
		this.id = id;
		this.imagePath = image_path;
		this.what = what;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public String getImagePath() {
		return imagePath;
	}

	public void setImagePath(String image_path) {
		this.imagePath = image_path;
	}

	public Type getWhat() {
		return what;
	}

	public void setWhat(Type what) {
		this.what = what;
	}

	public Bitmap getBitmap(Context c) {
		if (image == null) {
			image = ServerUtil.getBitmap(imagePath, c);
			if (imageDrawable == null) {
				imageDrawable = new BitmapDrawable(image);
			}
		}
		return image;
	}

	public Drawable getDrawable(Context c) {
		if (imageDrawable == null) {
			imageDrawable = (BitmapDrawable) ServerUtil.getDrawable(imagePath, c);
			if (image == null) {
				image = imageDrawable.getBitmap();
			}
		}
		return imageDrawable;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}
}
