package net.porkbuns.my_story.api;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import android.util.Log;

public class Quest {
	public static enum Status {
		ACTIVE, COMPLETED, NEW
	};

	private static HashMap<Integer, Quest> cache = new HashMap<Integer, Quest>();

	private int id;
	private String title;
	private String description;
	private LinkedList<Objective> objectives;
	private LinkedList<Item> rewards;
	private Status status;

	public static Quest get(int id) {
		if (cache.containsKey(new Integer(id))) {
			return cache.get(new Integer(id));
		} else {
			Document d = ServerUtil.fetchXml("/quests/" + id + ".xml");
			if (d != null) {
				return parseXml(d.getDocumentElement());
			}
			else {
				return null;
			}
		}
	}

	public static LinkedList<Quest> search(String query) {
		LinkedList<Quest> result = new LinkedList<Quest>();
		Map<String, String> params = new HashMap<String, String>();
		params.put("query", query);
		Document d = ServerUtil.fetchXml(ServerUtil.RequestType.GET, "/quests.xml", params);
		
		if (d != null) {
			NodeList nodes = d.getElementsByTagName("quest");
			int listLength = nodes.getLength();
			for (int i = 0; i < listLength; i++) {
				Log.d("API.Quest", nodes.item(i).getNodeName());
				result.add(parseXml(nodes.item(i)));
			}
			return result;
		}
		else {
			return null;
		}
	}

	public static LinkedList<Quest> getAll() {
		LinkedList<Quest> result = new LinkedList<Quest>();

		Document d = ServerUtil.fetchXml("/quests.xml");
		if (d != null) {
			NodeList nodes = d.getElementsByTagName("quest");
			int listLength = nodes.getLength();
			for (int i = 0; i < listLength; i++) {
				Log.d("API.Quest", nodes.item(i).getNodeName());
				result.add(parseXml(nodes.item(i)));
			}
	
			return result;
		}
		else {
			return null;
		}
	}

	private static Quest parseXml(Node e) {
		int id = 0;
		String title = null;
		String description = null;
		LinkedList<Objective> objectives = null;
		LinkedList<Item> rewards = null;

		NodeList nl = e.getChildNodes();
		Node n;
		int listLength = nl.getLength();

		for (int idx = 0; idx < listLength; idx++) {
			n = nl.item(idx);
			Log.d("API.Quest", n.getNodeName());
			String nodeName = n.getNodeName();

			if (nodeName.equals("quest-objectives")) {
				objectives = new LinkedList<Objective>();
				NodeList l = n.getChildNodes();
				int childLength = l.getLength();
				for (int i = 0; i < childLength; i++) {
					if (l.item(i).getNodeName().equals("quest-objective")) {
						objectives.add(Objective.parseXml(l.item(i)));
					}
				}
			} else if (nodeName.equals("rewards")) {
				rewards = new LinkedList<Item>();
				NodeList l = n.getChildNodes();
				int childLength = l.getLength();
				for (int i = 0; i < childLength; i++) {
					if (l.item(i).getNodeName().equals("reward")) {
						rewards.add(Item.parseXml(l.item(i)));
					}
				}
			} else {
				if (n.getFirstChild() == null)
					continue;

				String text = n.getFirstChild().getNodeValue();
				if (nodeName.equals("id")) {
					id = Integer.parseInt(text, 10);
					if (cache.containsKey(new Integer(id))) {
						return cache.get(new Integer(id));
					}
				} else if (nodeName.equals("description")) {
					description = text;
				} else if (nodeName.equals("title")) {
					title = text;
				}
			}
		}
		Quest q = new Quest(id, title, description, objectives, rewards);
		cache.put(new Integer(id), q);

		return q;
	}

	public static class Objective {
		private int id;
		private int questid;
		private double latitude;
		private double longitude;
		private int seconds;
		private int originalseconds;
		private boolean completed;
		private String description;
		private String name;
		private static HashMap<Integer, Objective> objCache = new HashMap<Integer, Objective>();

		public static Objective get(int objId, int questId) {
			if (objCache.containsKey(new Integer(objId))) {
				return objCache.get(new Integer(objId));
			} else {
				Quest.get(questId);
				return objCache.get(new Integer(objId));
			}
		}

		// (1,1, 37.87549, -122.25977, 5)

		static Objective parseXml(Node node) {
			int id = 0, seconds = 0, questid = 0;
			double latitude = 0, longitude = 0;
			String description = null, name = null;

			NodeList nl = node.getChildNodes();
			Node n;
			int listLength = nl.getLength();
			for (int idx = 0; idx < listLength; idx++) {
				n = nl.item(idx);
				if (n.getFirstChild() == null)
					continue;
				String text = n.getFirstChild().getNodeValue();
				String nodeName = n.getNodeName();

				if (nodeName.equals("latitude")) {
					latitude = Double.parseDouble(text);
				} else if (nodeName.equals("longitude")) {
					longitude = Double.parseDouble(text);
				} else if (nodeName.equals("seconds")) {
					seconds = Integer.parseInt(text, 10);
				} else if (nodeName.equals("id")) {
					id = Integer.parseInt(text, 10);
					if (objCache.containsKey(new Integer(id))) {
						return objCache.get(new Integer(id));
					}
				} else if (nodeName.equals("quest-id")) {
					questid = Integer.parseInt(text, 10);
				} else if (nodeName.equals("description")) {
					description = text;
				} else if (nodeName.equals("name")) {
					name = text;
				}
			}
			Objective o = new Objective(id, questid, latitude, longitude,
					seconds, description, name);
			objCache.put(new Integer(id), o);
			return o;
		}

		/**
		 * @param id
		 * @param questid
		 * @param latitude
		 * @param longitude
		 * @param seconds
		 */
		public Objective(int id, int questid, double latitude,
				double longitude, int seconds, String description, String name) {
			this.id = id;
			this.questid = questid;
			this.latitude = latitude;
			this.longitude = longitude;
			this.seconds = this.originalseconds = seconds;
			this.completed = false;
			this.description = description;
			this.name = name;
		}

		public String getDescription() {
			return description;
		}

		public void setDescription(String description) {
			this.description = description;
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public int getId() {
			return id;
		}

		public int getQuestId() {
			return questid;
		}

		/**
		 * @return the latitude
		 */
		public double getLatitude() {
			return latitude;
		}

		/**
		 * @return the longitude
		 */
		public double getLongitude() {
			return longitude;
		}

		/**
		 * @return the seconds
		 */
		public int getSeconds() {
			return seconds;
		}

		public int getOriginalSeconds() {
			return originalseconds;
		}

		public void resetSeconds() {
			this.seconds = originalseconds;
		}

		public void decrementSeconds() {
			seconds -= 1;
		}

		public boolean getCompleted() {
			return completed;
		}

		public void setCompleted(boolean completion) {
			completed = completion;
		}

	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * @return the objectives
	 */
	public LinkedList<Objective> getObjectives() {
		return objectives;
	}

	public int getNumIncompleteObj() {
		int objSize = objectives.size(), incompleteCount = 0;

		for (int i = 0; i < objSize; i++) {
			if (!objectives.get(i).completed) {
				incompleteCount++;
			}
		}

		return incompleteCount;
	}

	/**
	 * @return the rewards
	 */
	public LinkedList<Item> getRewards() {
		return rewards;
	}

	public Status getStatus() {
		return status;
	}

	public void setStatus(Status newStatus) {
		status = newStatus;
	}

	/**
	 * @param id
	 * @param title
	 * @param description
	 * @param objectives
	 * @param rewards
	 */
	private Quest(int id, String title, String description,
			LinkedList<Objective> objectives, LinkedList<Item> rewards) {
		this.id = id;
		this.title = title;
		this.description = description;
		this.objectives = objectives;
		this.rewards = rewards;
		this.status = Status.NEW;
	}
}
