package net.porkbuns.my_story.api;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpConnection;
import org.apache.commons.httpclient.HttpConnectionManager;
import org.apache.commons.httpclient.HttpMethodBase;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.HttpURL;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.SimpleHttpConnectionManager;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.util.Log;

public class ServerUtil {
	public static enum RequestType {
		GET, POST, PUT, DELETE
	};

	public static final String base_url = "http://ieee.berkeley.edu:3001";

	public static Document fetchXml(String url) {
		return fetchXml(RequestType.GET, url);
	}

	public static Document fetchXml(RequestType type, String url) {
		return fetchXml(type, url, new HashMap<String, String>(0));
	}

	public static Document fetchXml(RequestType type, String url,
			Map<String, String> params) {
		try {
			String response = httpRequest(type, url, params);
			if (response == null) {
				return null;
			}
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			return db.parse(new InputSource(new StringReader(response)));
		} catch (IOException ioe) {
			Log.e("ServerUtil", ioe.getMessage(), ioe);
		} catch (ParserConfigurationException pce) {
			Log.e("ServerUtil", pce.getMessage(), pce);
		} catch (SAXException se) {
			Log.e("ServerUtil", se.getMessage(), se);
		}
		return null;
	}

	public static String httpRequest(RequestType type, String url,
			Map<String, String> params) {
		
		url = base_url + url;
		try {
			Log.v("ServerUtil", "Loading: " + url);
			HttpURL httpURL = new HttpURL(url);
			HostConfiguration host = new HostConfiguration();
			host.setHost(httpURL.getHost(), httpURL.getPort());
			HttpConnectionManager connectionManager = new SimpleHttpConnectionManager();
			HttpConnection connection = connectionManager.getConnection(host);
			connection.open();

			HttpMethodBase method;
			if (type == RequestType.GET) {
				method = new GetMethod(url);
				NameValuePair[] p = new NameValuePair[params.size()];
				int i = 0;
				for (Map.Entry<String, String> e : params.entrySet()) {
					p[i++] = new NameValuePair(e.getKey(), e.getValue());
				}
				method.setQueryString(p);
			} else if (type == RequestType.POST) {
				method = new PostMethod(url);
				for (Map.Entry<String, String> entry : params.entrySet()) {
					((PostMethod) method).addParameter(entry.getKey(), entry
							.getValue());
				}
			} else if (type == RequestType.PUT) {
				method = new PutMethod(url);
				NameValuePair[] entity = new NameValuePair[params.size()];
				int idx = 0;
				for (Map.Entry<String, String> entry : params.entrySet()) {
					entity[idx] = new NameValuePair(entry.getKey(), entry.getValue());
					idx++;
				}
				method.setQueryString(entity);
			} else {
				method = new DeleteMethod(url);
			}
			method.execute(new HttpState(), connection);
			String response = method.getResponseBodyAsString();
			connection.close();
			return response;
		} catch (Exception e) {
			Log.e("ServerUtil", e.getMessage(), e);
		}
		return null;
	}

	public static Bitmap getBitmap(String url, Context c) {
		String fileName = url.replaceAll("/", "");
		File f = c.getFileStreamPath(fileName);

		if (f.exists()) {
			try {
				FileInputStream fin = c.openFileInput(fileName);
				Bitmap currentBitmap = BitmapFactory.decodeStream(fin);
				return currentBitmap;
			} catch (FileNotFoundException e) {
				Log.e("ServerUtil", e.getMessage());
			}
		} else {
			try {
				Log.v("ServerUtil", "Loading Bitmap: " + url);
				URL u = new URL(base_url + url);
				URLConnection connection = u.openConnection();
				InputStream is = connection.getInputStream();
				Bitmap bitmap = BitmapFactory.decodeStream(is);
				is.close();

				FileOutputStream fout = c.openFileOutput(fileName,
						Context.MODE_WORLD_READABLE);
				if (fileName.contains("png")) {
					bitmap.compress(CompressFormat.PNG, 100, fout);
				} else {
					bitmap.compress(CompressFormat.JPEG, 100, fout);
				}
				fout.close();

				return bitmap;
			} catch (Exception e) {
				Log.e("ServerUtil", e.getMessage());
			}
		}
		return null;
	}

	public static Drawable getDrawable(String url, Context c) {
		return new BitmapDrawable(getBitmap(url, c));
	}
}
