package net.porkbuns.my_story.api;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import net.porkbuns.my_story.QuestDBAdapter;
import net.porkbuns.my_story.api.ServerUtil.RequestType;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import android.content.Context;
import android.database.Cursor;
import android.util.Log;
import android.view.View;

public class User {
	public static final int MALE = 0;
	public static final int FEMALE = 1;
	public static final int ROOMSIZE = 4;
	private static final double STARTING_LATITUDE = 37.87629308319713;
	private static final double STARTING_LONGITUDE = -122.25656747817993;
	private static final String AVATAR_FILE_NAME = "avatarData.txt";
	private final static String ROOM_FILE_NAME = "roomData.txt";

	private int id;
	private String email;
	private boolean male;
	private Item head;
	private Item shirt;
	private Item pants;
	private int headPos;
	private int shirtPos;
	private int pantsPos;
	private long phoneNumber;
	private ArrayList<Quest> completedQuests;
	private ArrayList<Quest> activeQuests;
	private ArrayList<Quest.Objective> completedObj;
	private ArrayList<Quest.Objective> activeObj;
	private ArrayList<Item> headItems;
	private ArrayList<Item> shirtItems;
	private ArrayList<Item> pantsItems;
	private ArrayList<Item> inventory;
	private Item[] roomItems;
	//private int[] roomItemIds = { 0, 0, 0, 0 };

	private double longitude;
	private double latitude;

	private static User currentUser = null;

	public static User getCurrentUser(Context c) {
		if (currentUser == null) {
			currentUser = User.get(loadPhoneNumber(c));
			if (currentUser != null) {
				User.loadQuests(currentUser, c);
				currentUser.latitude = STARTING_LATITUDE;
				currentUser.longitude = STARTING_LONGITUDE;
			}
		}
		return currentUser;
	}

	public static User create(String email, long phoneNumber, boolean male,
			Context c) {
		Map<String, String> params = new HashMap<String, String>();
		params.put("user[phone]", "" + phoneNumber);
		params.put("user[email]", email);
		params.put("user[male]", male ? "1" : "0");
		ServerUtil.httpRequest(RequestType.POST, "/users.xml", params);

		User u = new User(0, email, male, null, null, null, phoneNumber, null,
				null, new ArrayList<Item>(), new Item[] { null, null, null, null });
		u.writeAvatarData(c);
		u.writeRoomData(c);
		return getCurrentUser(c);
	}

	protected static long loadPhoneNumber(Context c) {
		long[] fileData = { 0, 0, 0, 0, 0 };

		File file = c.getFileStreamPath(AVATAR_FILE_NAME);
		if (file.exists()) {
			try {
				BufferedReader reader = new BufferedReader(new FileReader(file));

				// Read a line of text and parse it
				String s = reader.readLine();

				Log.v("User", "avatar file data: " + s);
				String[] strArr = s.split(",");

				for (int i = 0; i < strArr.length; i++) {
					fileData[i] = Long.parseLong(strArr[i], 10);
				}

				reader.close();
			}
			// Catches any error conditions
			catch (IOException e) {
				Log.e("User", e.getMessage());
			}
		}

		// you know, we only care about phone number at first... make it so.
		return fileData[0];
	}

	public static void loadQuests(User u, Context c) {
		QuestDBAdapter mAdapter = new QuestDBAdapter(c);
		mAdapter.open();

		u.activeQuests = new ArrayList<Quest>();
		u.completedQuests = new ArrayList<Quest>();
		u.activeObj = new ArrayList<Quest.Objective>();
		u.completedObj = new ArrayList<Quest.Objective>();

		boolean[] values = new boolean[2];
		values[0] = false;
		values[1] = true;

		for (int j = 0; j < 2; j++) {
			Cursor cur = mAdapter.fetchAllQuests(values[j]);
			if (cur != null) {
				do {
					int id = cur.getInt(mAdapter.INDEX_QUESTID);
					Quest q = Quest.get(id);

					Cursor obj = mAdapter.fetchObjectiveGivenQuestId(q.getId());

					if (obj != null) {
						LinkedList<Quest.Objective> questObjectives = q
								.getObjectives();
						int objSize = questObjectives.size();

						for (int i = 0; i < objSize; i++) {
							int currObjStatus = obj
									.getInt(mAdapter.INDEX_OBJSTATUS);
							Quest.Objective currObjective = questObjectives
									.get(i);
							currObjective.setCompleted(currObjStatus == 1);
							if (currObjStatus == 1) {
								u.completedObj.add(currObjective);
							} else {
								u.activeObj.add(currObjective);
							}
							obj.next();
						}
					}

					if (values[j]) {
						q.setStatus(Quest.Status.COMPLETED);
						u.completedQuests.add(q);
					} else {
						q.setStatus(Quest.Status.ACTIVE);
						u.activeQuests.add(q);
					}

				} while (cur.next());
				cur.close();
			}
		}

		mAdapter.close();
	}

	public String save(Context c) {
		String response = ServerUtil.httpRequest(RequestType.PUT, getUrl(), buildHttpParameters());
		writeAvatarData(c);
		writeRoomData(c);
		
		return response;
	}

	private Map<String, String> buildHttpParameters() {
		Map<String, String> r = new HashMap<String, String>(5);
		r.put("user[email]", email);
		r.put("user[male]", male ? "1" : "0");
		r.put("user[head_id]", head.getId() + "");
		r.put("user[shirt_id]", shirt.getId() + "");
		r.put("user[pant_id]", pants.getId() + "");
		
		for (int i = 1; i <= 4; i++) {
			Item currItem = roomItems[i-1];
			if (currItem != null) {
				r.put("user[room" + i + "]", currItem.getId() + "");
			}
			else {
				r.put("user[room" + i + "]","-1");
			}
		}
		return r;
	}

	private static String getUrl(long phoneNumber) {
		return "/users/" + phoneNumber + ".xml";
	}

	private String getUrl() {
		return getUrl(getPhoneNumber());
	}

	public void writeAvatarData(Context c) {
		FileOutputStream fout;
		try {
			// Open an output stream
			fout = c.openFileOutput(AVATAR_FILE_NAME,
					Context.MODE_WORLD_WRITEABLE);
			PrintStream out = new PrintStream(fout);
			if (male) {
				out.print(phoneNumber + "," + 0 + "," + headPos + ","
						+ shirtPos + "," + pantsPos + "\n");
			} else {
				out.print(phoneNumber + "," + 1 + "," + headPos + ","
						+ shirtPos + "," + pantsPos + "\n");
			}

			// Close our output stream
			fout.close();
			out.close();
		}
		// Catches any error conditions
		catch (IOException e) {
			Log.e("User", e.getMessage());
		}
	}

	public void writeRoomData(Context c) {
		FileOutputStream fout;
		try {
			// Open an output stream
			fout = c.openFileOutput(ROOM_FILE_NAME,
					Context.MODE_WORLD_WRITEABLE);
			PrintStream out = new PrintStream(fout);

			for (int i = 0; i < ROOMSIZE; i++) {
				Item currItem =roomItems[i];
				if (currItem != null) {
					out.print(currItem.getId());
				}
				else {
					out.print(-1);
				}
				if (i != ROOMSIZE - 1) {
					out.print(",");
				}
			}
			out.print("\n");

			// Close our output stream
			fout.close();
			out.close();
		}
		// Catches any error conditions
		catch (IOException e) {
			Log.e("User", e.getMessage());
		}
	}

	public static User get(long phoneNumber) {
		Document d = ServerUtil.fetchXml(getUrl(phoneNumber));
		if (d == null) {
			return null;
		}
		NodeList nl = d.getElementsByTagName("user");
		if (nl.getLength() > 0) {
			return parseXml(nl.item(0));
		}
		else {
			return null;
		}
	}

	private static User parseXml(Node e) {
		String email = "";
		int id = 0;
		long phoneNumber = 0;
		Item head = null, shirt = null, pant = null;
		boolean male = true;
		ArrayList<Quest> completedQuests = new ArrayList<Quest>();
		ArrayList<Quest> activeQuests = new ArrayList<Quest>();
		ArrayList<Item> items = new ArrayList<Item>();
		Item[] roomItems = new Item[4];

		NodeList nl = e.getChildNodes();
		Node n;
		int listLength = nl.getLength();

		for (int idx = 0; idx < listLength; idx++) {
			n = nl.item(idx);
			if (n.getFirstChild() == null)
				continue;
			String text = n.getFirstChild().getNodeValue();
			String nodeName = n.getNodeName();

			if (nodeName.equals("id")) {
				id = Integer.parseInt(text, 10);
			} else if (nodeName.equals("email")) {
				email = text;
			} else if (nodeName.equals("head-id")) {
				head = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("male")) {
				male = text.equals("true");
			} else if (nodeName.equals("pant-id")) {
				pant = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("phone")) {
				phoneNumber = Long.parseLong(text, 10);
			} else if (nodeName.equals("shirt-id")) {
				shirt = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("room1")) {
				roomItems[0] = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("room2")) {
				roomItems[1] = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("room3")) {
				roomItems[2] = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("room4")) {
				roomItems[3] = Item.get(Integer.parseInt(text, 10));
			} else if (nodeName.equals("items")) {
				int itemIdx = 0;
				int numItems = n.getChildNodes().getLength();
				
				for (NodeList itemNodes = n.getChildNodes(); itemIdx < numItems; itemIdx++) {
					Node item = itemNodes.item(itemIdx);
					if (item.getNodeType() == Node.ELEMENT_NODE && item.getNodeName().equals("item")) {
						items.add(Item.parseXml(item));
					}
				}
			}
		}

		return new User(id, email, male, head, shirt, pant, phoneNumber,
				completedQuests, activeQuests, items, roomItems);
	}

	public static boolean hasAccount(Context c) {
		File file = c.getFileStreamPath(AVATAR_FILE_NAME);
		try {
			if (file.exists()) {
				BufferedReader reader = new BufferedReader(new FileReader(file));
				String s = reader.readLine();
				if (s == null) {
					return false;
				}
				return true;
			}
		} catch (IOException e) {
			Log.e("User", e.getMessage());
		}
		return false;
	}

	/**
	 * @param id
	 * @param email
	 * @param male
	 * @param head
	 * @param shirt
	 * @param pant
	 * @param phoneNumber
	 * @param completedQuests
	 * @param activeQuests
	 * @param items
	 */
	private User(int id, String email, boolean male, Item head, Item shirt,
			Item pants, long phoneNumber, ArrayList<Quest> completedQuests,
			ArrayList<Quest> activeQuests, ArrayList<Item> inventory,
			Item[] roomItems) {
		this.id = id;
		this.email = email;
		this.male = male;
		this.head = head;
		this.shirt = shirt;
		this.pants = pants;
		this.phoneNumber = phoneNumber;
		this.completedQuests = completedQuests;
		this.activeQuests = activeQuests;
		this.inventory = inventory;
		this.roomItems = roomItems;

		headItems = new ArrayList<Item>();
		Item.getAll(male, Item.Type.head);
		shirtItems = new ArrayList<Item>();
		Item.getAll(male, Item.Type.shirt);
		pantsItems = new ArrayList<Item>();
		Item.getAll(male, Item.Type.pants);
		
		for (Item i : inventory) {
			Item.Type w = i.getWhat();
			if (w == Item.Type.head) {
				headItems.add(i);
			} else if (w == Item.Type.shirt) {
				shirtItems.add(i);
			} else if (w == Item.Type.pants) {
				pantsItems.add(i);
			}
		}
		
		this.headPos = headItems.indexOf(this.head);
		this.shirtPos = shirtItems.indexOf(this.shirt);
		this.pantsPos = pantsItems.indexOf(this.pants);
	}

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	/**
	 * @return the email
	 */
	public String getEmail() {
		return email;
	}

	public void setEmail(String emailAddr) {
		email = emailAddr;
	}

	/**
	 * @return the male
	 */
	public boolean isMale() {
		return male;
	}

	/**
	 * @return the head
	 */
	public Item getHead() {
		return head;
	}

	public int getHeadPos() {
		return headPos;
	}

	public ArrayList<Item> getHeadItems() {
		return headItems;
	}

	public void setHead(int pos) {
		head = headItems.get(pos);
		headPos = pos;
	}

	/**
	 * @return the shirt
	 */
	public Item getShirt() {
		return shirt;
	}

	public int getShirtPos() {
		return shirtPos;
	}

	public ArrayList<Item> getShirtItems() {
		return shirtItems;
	}

	public void setShirt(int pos) {
		shirt = shirtItems.get(pos);
		shirtPos = pos;
	}

	/**
	 * @return the pant
	 */
	public Item getPants() {
		return pants;
	}

	public int getPantsPos() {
		return pantsPos;
	}

	public ArrayList<Item> getPantsItems() {
		return pantsItems;
	}

	public void setPants(int pos) {
		pants = pantsItems.get(pos);
		pantsPos = pos;
	}

	/**
	 * @return the phone_number
	 */
	public long getPhoneNumber() {
		return phoneNumber;
	}

	public void setPhoneNumber(long phone) {
		phoneNumber = phone;
	}

	/**
	 * @return the completed_quests
	 */
	public ArrayList<Quest> getCompletedQuests() {
		return completedQuests;
	}

	/**
	 * @return the active_quests
	 */
	public ArrayList<Quest> getActiveQuests() {
		return activeQuests;
	}

	public int getNumCompleted() {
		return completedQuests.size();
	}

	public int getNumActive() {
		return activeQuests.size();
	}

	public ArrayList<Quest.Objective> getCompletedObj() {
		return completedObj;
	}

	public ArrayList<Quest.Objective> getActiveObj() {
		return activeObj;
	}

	// TODO: replace with server acceptance
	public void acceptQuest(Quest accepted, Context c) {
		QuestDBAdapter mAdapter = new QuestDBAdapter(c);
		mAdapter.open();
		if (!activeQuests.contains(accepted)
				&& !completedQuests.contains(accepted)) { // quest not in
			// database
			mAdapter.createQuest(accepted.getId(), false);
			accepted.setStatus(Quest.Status.ACTIVE);

			LinkedList<Quest.Objective> objectives = accepted.getObjectives();
			int objSize = objectives.size();
			for (int i = 0; i < objSize; i++) {
				Quest.Objective o = objectives.get(i);
				mAdapter.createObjective(o.getId(), accepted.getId(), 0);
				activeObj.add(o);
			}

			activeQuests.add(accepted);
		}

		mAdapter.close();
		
		HashMap<String, String> params = new HashMap<String, String>(1);
		params.put("user_quest[quest_id]", accepted.getId() + "");
		ServerUtil.httpRequest(RequestType.POST, "/users/"+ phoneNumber+"/quests.xml", params);
	}

	// TODO: replace with server completion
	public void completeObjective(Quest.Objective completed, Context c) {
		QuestDBAdapter mAdapter = new QuestDBAdapter(c);
		Quest q = Quest.get(completed.getQuestId());
		completed.setCompleted(true);
		activeObj.remove(completed);
		completedObj.add(completed);

		mAdapter.open();
		mAdapter.updateObjectiveStatus(completed.getId(), 1);
		mAdapter.close();

		if (q.getNumIncompleteObj() == 0) {
			completeQuest(q, c);
		}
	}

	// TODO: replace with server completion
	public void completeQuest(Quest completed, Context c) {
		QuestDBAdapter mAdapter = new QuestDBAdapter(c);
		mAdapter.open();
		Cursor quest = mAdapter.fetchQuestGivenId(completed.getId(), false);
		if (quest != null) {
			mAdapter.deleteQuest(completed.getId(), false);
			mAdapter.createQuest(completed.getId(), true);
			quest.close();
		}
		activeQuests.remove(completed);
		completedQuests.add(completed);
		completed.setStatus(Quest.Status.COMPLETED);

		// Add rewards as result of completing quest
		LinkedList<Item> rewards = completed.getRewards();
		int numRewards = rewards.size();
		for (int i = 0; i < numRewards; i++) {
			Item currReward = rewards.get(i);
			ArrayList<Item> listToAdd;
			Item.Type itemType = currReward.getWhat();

			// add to the correct list based on the type
			if (itemType == Item.Type.head) {
				listToAdd = headItems;
			} else if (itemType == Item.Type.shirt) {
				listToAdd = shirtItems;
			} else if (itemType == Item.Type.pants) {
				listToAdd = pantsItems;
			} else {
				listToAdd = inventory;
			}

			if (!listToAdd.contains(currReward)) {
				listToAdd.add(currReward);
			}
		}

		mAdapter.close();
		
		ServerUtil.httpRequest(RequestType.POST, "/users/"+ phoneNumber+"/quests/"+completed.getId()+"/complete.xml", new HashMap<String, String>(0));
	}

	/**
	 * @return the items
	 */
	public ArrayList<Item> getTrophies() {
		ArrayList<Item> r = new ArrayList<Item>();
		for (Item i : inventory) {
			if (i.getWhat() == Item.Type.trophy) r.add(i);
		}
		return r;
	}
	
	public ArrayList<Item> getItems() {
		return inventory;
	}

	public View getAvatarView() {
		return null;
	}

	public ArrayList<RecentEvent> getRecentEvents() {
		return null;
	}

	public Item[] getRoomItems() {
		return roomItems;
	}

	public Item getRoomItem(int pos) {
		return roomItems[pos];
	}

	public void updateRoom(int pos, int id, Context c) {
		for (int i = 0; i < ROOMSIZE; i++) {
			Item currItem = roomItems[i];
			if (currItem != null && currItem.getId() == id && id != -1) {
				swapItems(i, pos, Item.get(id));
				return;
			}
		}

		if (id != -1) {
			roomItems[pos] = Item.get(id);
		}

		writeRoomData(c); // so when it say "changes saved", its *actually*
		// saving the changes!
	}

	public void swapItems(int oldPos, int newPos, Item newItem) {
		roomItems[oldPos] = roomItems[newPos];
		roomItems[newPos] = newItem;
	}

	public double getLatitude() {
		return latitude;
	}

	public double getLongitude() {
		return longitude;
	}

	public void changeLatitude(double adjustment) {
		latitude += adjustment;
	}

	public void changeLongitude(double adjustment) {
		longitude += adjustment;
	}

}
